/* This file is part of the ili2ora project.
 * For more information, please see <http://www.interlis.ch>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package ch.ehi.sqlgen.generator;

import java.util.HashMap;
import java.util.ArrayList;
import java.util.Iterator;

import java.io.IOException;

import java.sql.Connection;
import java.sql.Statement;
import java.sql.SQLException;

import ch.ehi.basics.logging.EhiLogger;
import ch.ehi.sqlgen.generator.Generator;
import ch.ehi.sqlgen.generator.SqlConfiguration;
import ch.ehi.sqlgen.repository.*;

/**
 * @author ce
 * @version $Revision: 1.0 $ $Date: 09.02.2005 $
 */
public class GeneratorOracle implements Generator {
	private java.io.StringWriter out=null;
	private Connection conn=null;
	private ArrayList dropLines=null; // list<AbstractStmt>
	private ArrayList createLines=null;  // list<AbstractStmt>
	public abstract class AbstractStmt {
		private String line=null;
		public AbstractStmt(String line1){
			line=line1;
		}
		public String getLine(){
			return line;
		}
	}
	public class Stmt extends AbstractStmt {
		public Stmt(String line1){
			super(line1);
		}
	}
	public class Cmt extends AbstractStmt{
		public Cmt(String line1){
			super(line1);
		}
	}
	public GeneratorOracle(){
	}
	public void visitSchemaBegin(SqlConfiguration config, DbSchema schema)
		throws IOException {
		conn=config.getJdbcConnection();
		if(conn==null){
			throw new IllegalArgumentException("config.getConnection()==null");
		}
		dropLines=new ArrayList();
		createLines=new ArrayList();
	}

	public void visitSchemaEnd(DbSchema schema) throws IOException {
	}

	public void visit1Begin() throws IOException {
	}

	public void visit1End() throws IOException {
	}

	public void visit2Begin() throws IOException {
	}

	public void visit2End() throws IOException {
	}
	
	private String colSep=null;
	public void visit1TableBegin(DbTable tab) throws IOException {
		out = new java.io.StringWriter();
		String tabName=tab.getName();
		out.write(getIndent()+"CREATE TABLE "+tabName+" ("+newline());
		inc_ind();
		addDropLine(new Cmt(tab.getIliName()));
		addDropLine(new Stmt("DROP TABLE "+tabName));
		colSep="";
	}

	public void visit1TableEnd(DbTable tab) throws IOException {
		dec_ind();
		out.write(getIndent()+")"+newline());
		// execute stmt
		String stmt=out.toString();
		addCreateLine(new Cmt(tab.getIliName()));
		addCreateLine(new Stmt(stmt));
		EhiLogger.traceBackendCmd(stmt);
		out=null;
		Statement dbstmt = null;
		try{
			try{
				dbstmt = conn.createStatement();
				dbstmt.executeUpdate(stmt);
			}finally{
				dbstmt.close();
			}
		}catch(SQLException ex){
			EhiLogger.logError("failed to create table "+tab.getName(),ex);
		}
		
	}

	public void visit2TableBegin(DbTable arg0) throws IOException {
	}
	public void visit2TableEnd(DbTable arg0) throws IOException {
	}
	public void visitColumn(DbColumn column) throws IOException {
		String type="";
		String size="";
		String notSupported=null;
		if(column instanceof DbColBoolean){
			type="NUMBER(1)";
		}else if(column instanceof DbColDateTime){
			type="TIMESTAMP";
		}else if(column instanceof DbColDecimal){
			DbColDecimal col=(DbColDecimal)column;
			type="DECIMAL("+Integer.toString(col.getSize())+","+Integer.toString(col.getPrecision())+")";
		}else if(column instanceof DbColGeometry){
			type="MDSYS.SDO_GEOMETRY";
		}else if(column instanceof DbColId){
			type="NUMBER(9)";
		}else if(column instanceof DbColNumber){
			DbColNumber col=(DbColNumber)column;
			type="NUMBER("+Integer.toString(col.getSize())+")";
		}else if(column instanceof DbColVarchar){
			int colsize=((DbColVarchar)column).getSize();
			type="VARCHAR2("+Integer.toString(colsize)+")";
		}else{
			type="VARCHAR2(20)";
		}
		String isNull=column.isNotNull()?"NOT NULL":"NULL";
		String name=column.getName();
		out.write(getIndent()+colSep+name+" "+type+" "+isNull+newline());
		colSep=",";
	}
	public void visitTableBeginColumn(DbTable arg0) throws IOException {
	}
	public void visitTableEndColumn(DbTable arg0) throws IOException {
	}
	public void visitIndex(DbIndex arg0) throws IOException {
	}
	public void visitTableBeginIndex(DbTable arg0) throws IOException {
	}
	public void visitTableEndIndex(DbTable arg0) throws IOException {
	}
	public void visitConstraint(DbConstraint arg0) throws IOException {
	}
	public void visitTableBeginConstraint(DbTable arg0) throws IOException {
	}
	public void visitTableEndConstraint(DbTable arg0) throws IOException {
	}
	public void visitEnumEle(DbEnumEle arg0) throws IOException {
	}
	public void visitTableBeginEnumEle(DbTable arg0) throws IOException {
	}
	public void visitTableEndEnumEle(DbTable arg0) throws IOException {
	}
	// utilities
	private String getIndent(){
		return "";
	}
	private void inc_ind()
	{
	}
	private void dec_ind()
	{
	}
	private String newline(){
		return "";
	}
	private void addDropLine(AbstractStmt dropstmt){
		dropLines.add(dropstmt);
	}
	public Iterator iteratorDropLines(){
		return dropLines.iterator();
	}
	private void addCreateLine(AbstractStmt createstmt){
		createLines.add(createstmt);
	}
	public Iterator iteratorCreateLines(){
		return createLines.iterator();
	}
}
